/* File: MPI_Fault.c
 * 
 * Hatem Nassrat
 * B00393388 - Dalhousie University
 *
 * A Fault tolerance Implementation for an OLAP system
 *
 */

#include <mpi.h>
#include <stdio.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "defines.h"

#define IC_CREATE_TAG 5

Element* get_my_records(int n, int p_rank, int p_size);
void disk_store(char * fname, Element* array, int n);

int main(int argc, char * argv[])
{
    int i, my_rank, p, f;
    char buf_file[50];
    MPI_Comm *my_comm;
    Element* my_records;
    struct timeval start,stop;

    /*MPI INIT*/
    
    MPI_Init(&argc,&argv);
    MPI_Comm_rank(MPI_COMM_WORLD,&my_rank);
    MPI_Comm_size(MPI_COMM_WORLD, &p);
    f=p;
    /*END OF: MPI INIT*/

    if(argc>1) DATA_SIZE=atoi(argv[1]);
    
    gettimeofday(&start,NULL);
    /* Init data & compute backups */
    if(my_rank != 0)
    {
        my_records = get_my_records(DATA_SIZE,my_rank,p);
        disk_store("/tmp/pcc05/my_data",my_records,DATA_SIZE);
    }
    /* End of init and backup computation*/
    
    gettimeofday(&stop,NULL);
    if(my_rank==1)
        printf("init_data: Time Elapsed %.4f\n",
                (stop.tv_sec-start.tv_sec)+
                ((float)(stop.tv_usec-start.tv_usec))/(1000000));

  /* Finalize */
    MPI_Finalize();
    exit(0);
}

Element* get_my_records(int n, int p_rank, int p_size)
{
    int i;
    Element* array = (Element*)malloc(n*sizeof(Element));

    time_t now;
    (void) time(&now);
    srand48((long) now);
   

    for(i=0;i<n;i++)
    {
        array[i].key = i*(p_size-1)+(p_rank-1);
        array[i].data = lrand48()*.99;
    }
    
    //each processor has a set of sorted keys

    return array;
}

void disk_store(char *fname, Element* array, int n)
{
    FILE *f;
    int i;

    f= fopen(fname,"w");
 
    for(i=0;i<n;i++)
    {
        fprintf(f,"%d,%f\n",array[i].key,array[i].data);
    }
    
    fclose(f);
}

